
module cwx.cwls;

import std.stream;
import std.c.string;

import std.file;
import std.path;
import std.math;
import std.string;

import cwx.binary;
import cwx.sjis;

/// fからbyteを読込み、0以外ならtrueを、0ならfalseを返す。
private bool readBool(InputStream f) {
	byte b;
	f.read(b);
	return b ? true : false;
}
/// fから画像を読込む。
private byte[] readImage(InputStream f) {
	uint len = readUIntL(f);
	if (!len) return [];
	ubyte[] img;
	img.length = len;
	f.read(img);
	return cast(byte[]) img;
}
/// fから文字列を読込む。
/// lnsを指定した場合は末尾の0が存在する文字列、
/// cutTextを指定した場合はTEXT\r\nが先頭にある文字列
/// として読込む。
private string readString(InputStream f, bool lns = false, bool cutText = false) {
	uint len = readUIntL(f);
	if (!len) return "";
	char[] str;
	str.length = len;
	f.read(cast(ubyte[]) str);
	if (!lns && str[$ - 1] == '\0') str = str[0 .. $ - 1];
	string str2 = touni(cast(immutable) str);
	if (cutText) {
		str2 = str2.length > "TEXT\r\n".length ? str2["TEXT\r\n".length .. $] : "";
	}
	str2 = replace(str2, "\r\n", "\n");
	return str2;
}
/// 文字列を読込み、行毎に分割する。
private string[] readStrings(InputStream f) {
	auto str = readString(f, true);
	return str.length ? splitlines(str) : cast(string[]) [];
}

/// シナリオの概要。
struct Summary {
	/// シナリオのパス。
	/// 展開されたシナリオの場合はフォルダのパス。
	/// cab書庫の場合はcabファイルのパス。
	string sPath;
	byte[] image; /// 貼り紙画像。
	string name; /// シナリオ名。
	string desc; /// 解説。
	string author; /// 作成者。
	string[] rCoupons; /// 必要クーポン。
	uint rCouponNum; /// 必要クーポンの数。
	uint startAreaId; /// 開始エリア。
	uint levelMin; /// 対象レベル。
	uint levelMax; /// ditto
}

/// fからシナリオの概要を読込む。
Summary loadSummary(InputStream f, string sPath) {
	Summary summ;
	byte b;
	summ.sPath = sPath;
	summ.image = readImage(f);
	summ.name = readString(f);
	summ.desc = readString(f, true);
	summ.author = readString(f);
	summ.rCoupons = readStrings(f);
	summ.rCouponNum = readUIntL(f);
	summ.startAreaId = readUIntL(f) - 40000u;
	// Flag/Step読み捨て
	uint stepNum = readUIntL(f);
	for (uint i = 0u; i < stepNum; i++) {
		readString(f);
		readUIntL(f);
		for (uint j = 0u; j < 10u; j++) {
			readString(f);
		}
	}
	uint flagNum = readUIntL(f);
	for (uint i = 0u; i < flagNum; i++) {
		readString(f);
		readBool(f);
		readString(f);
		readString(f);
	}
	readUIntL(f);
	summ.levelMin = readUIntL(f);
	summ.levelMax = readUIntL(f);
	return summ;
}
